import json
import random
import redis


async def redis_client(host, port):
	redis_client = redis.Redis(
		host=host,
		port=port,
		decode_responses=True  
	)
	return redis_client


async def set_cache(key: str, value: dict, redis_client_data:dict, expire_minutes: float, allow_random: bool = False):
  try:
    r_client = await redis_client(redis_client_data["host"],redis_client_data["port"])
    r_client.set(name=key, value=json.dumps(value), ex=(expire_minutes*random.randint(60, 120) if allow_random else expire_minutes*60))

  except Exception as e:
    raise e


async def get_cache(key: str, redis_client_data:dict):
  try:
    r_client = await redis_client(redis_client_data["host"],redis_client_data["port"])
    value = r_client.get(key)
    return json.loads(value) if value else None

  except Exception as e:
    raise e


import asyncio
from functools import wraps
from uuid import uuid4

from aio_pika import ExchangeType, Message, connect_robust
from aio_pika.abc import AbstractIncomingMessage

APP, suffix, url, connection = ("", "", "", None)
registerd_consumers = []
futures = {}


def set_future_result(correlation_id: str, value):
  global futures
  if correlation_id in futures:
    future: asyncio.Future = futures.pop(correlation_id)
    future.set_result(value)


async def message_consumer(func, exchange_name: str | None = None, exchange_type: ExchangeType = ExchangeType.DIRECT, queue_name: str | None = None, routing_key: str | None = None, no_ack: bool = False, durable: bool = False, exclusive: bool = False,):
  global connection
  if connection is None:
    raise Exception("RabbitMQ connection not initialized")

  async with connection.channel() as channel:
    await channel.set_qos(prefetch_count=1)

    if queue_name is not None:
      queue = await channel.declare_queue(name=queue_name, durable=durable, exclusive=exclusive)
    else:
      queue = await channel.declare_queue(durable=durable, exclusive=exclusive)

    if exchange_name is not None:
      exchange = await channel.declare_exchange(exchange_name, exchange_type)

    if routing_key is not None:
      await queue.bind(exchange, routing_key=routing_key)
    else:
      await queue.bind(exchange)

    async with queue.iterator(no_ack=no_ack) as qiterator:
      message: AbstractIncomingMessage
      async for message in qiterator:
        await func(message)


async def publish(payload: bytes, exchange_name: str | None = None, exchange_type: ExchangeType = ExchangeType.DIRECT, callback_queue_name: str | None = None, routing_key: str = None, rpc: bool = False, correlation_id: str | None = None):
  global connection

  if connection is None: 
    raise Exception("RabbitMQ connection not initialized")
  
  new_routing_key = routing_key if routing_key is None else routing_key + suffix
  
  async with connection.channel() as channel:
    await channel.set_qos(prefetch_count=1)

    if exchange_name is not None:
      exchange = await channel.declare_exchange(exchange_name, exchange_type)
    else:
      exchange = channel.default_exchange

    if not rpc:
      await exchange.publish(Message(payload, correlation_id=correlation_id, app_id=APP), routing_key=new_routing_key)
    else:
      global futures

      if correlation_id is None:
        correlation_id = str(uuid4())
      
      future = asyncio.get_running_loop().create_future()

      futures[correlation_id] = future
      await exchange.publish(Message(payload, correlation_id=correlation_id, reply_to=callback_queue_name, content_type="text/plain", app_id=APP), routing_key=new_routing_key)

      return await future


def consumer(exchange_name: str | None = None, exchange_type: ExchangeType = ExchangeType.DIRECT, queue_name: str | None = None, routing_key: str | None = None, no_ack: bool = False, durable: bool = False, exclusive: bool = False,):
  def decorator(func):
    global registerd_consumers

    new_queue_name = queue_name if queue_name is None else queue_name + suffix
    new_routing_key = routing_key if routing_key is None else routing_key + suffix

    registerd_consumers.append({
      "exchange_name": exchange_name, 
      "exchange_type": exchange_type, 
      "queue_name": new_queue_name, 
      "routing_key": new_routing_key, 
      "func": func,
      "no_ack": no_ack,
      "durable": durable,
      "exclusive": exclusive
    })

    @wraps(func)
    async def wrapper():
      await message_consumer(func, exchange_name, exchange_type, new_queue_name, new_routing_key, no_ack, durable, exclusive)

    return wrapper
  
  return decorator



async def init():
  global connection

  if url is None:
    raise Exception("RABBITMQ_URL not set")
  
  connection = await connect_robust(url, loop=asyncio.get_running_loop())
  for c in registerd_consumers:
    asyncio.ensure_future(message_consumer(**c))


async def close():
  global connection
  if connection is not None:
    await connection.close()
  

def update_config(rabbitmq_setup_data: dict):
  global APP, suffix, url

  APP, suffix, url = rabbitmq_setup_data.get("app"), rabbitmq_setup_data.get("suffix"), rabbitmq_setup_data.get("url")
  if APP is None:
    raise Exception("APP not set")



import smtplib
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from azure.storage.blob import BlobServiceClient, ContentSettings
from fastapi import UploadFile
from uuid import uuid4

class CommonService():
  @staticmethod
  async def send_email(data):
    try:
      SMTP_SERVER = data.get("SMTP_SERVER")
      SMTP_PORT = data.get("SMTP_PORT")
      EMAIL = data.get("EMAIL")
      EMAIL_PASSWORD = data.get("EMAIL_PASSWORD")
      subject = data.get("subject")
      from_address = data.get("from_address")
      to_address = data.get("to_address")
      body = data.get("body")

      server = smtplib.SMTP(SMTP_SERVER, SMTP_PORT)
      server.starttls()
      server.ehlo()
      server.login(EMAIL,EMAIL_PASSWORD)

      msg = MIMEMultipart('alternative')
      msg['Subject'] = subject
      msg['From'] = from_address
      msg['To'] = to_address
      
      msg.attach(MIMEText(body, 'html'))
      server.sendmail(from_address, to_address, msg.as_string())
    except:
      raise

  @staticmethod
  async def save_file_to_azure(image_data, file: UploadFile):
    try:
      AZURE_STORAGE_ACCOUNT_NAME = image_data.get("AZURE_STORAGE_ACCOUNT_NAME")
      AZURE_STORAGE_ACCOUNT_KEY = image_data.get("AZURE_STORAGE_ACCOUNT_KEY")
      AZURE_CONTAINER_NAME = image_data.get("AZURE_CONTAINER_NAME")

      # Configure Azure Storage Blob
      blob_service_client = BlobServiceClient.from_connection_string(
          f"DefaultEndpointsProtocol=https;AccountName={AZURE_STORAGE_ACCOUNT_NAME};AccountKey={AZURE_STORAGE_ACCOUNT_KEY};EndpointSuffix=core.windows.net"
      )
      container_client = blob_service_client.get_container_client(AZURE_CONTAINER_NAME)
      unique_blob_name = f"{uuid4()}-{file.filename}"
      blob_client = container_client.get_blob_client(unique_blob_name)
      content = file.file.read()
      blob_client.upload_blob(content, content_settings=ContentSettings(content_type=file.content_type))
      return f"https://{AZURE_STORAGE_ACCOUNT_NAME}.blob.core.windows.net/{AZURE_CONTAINER_NAME}/{unique_blob_name}"
    except:
      raise



import certifi
from beanie import init_beanie
from motor.motor_asyncio import AsyncIOMotorClient

from base_utils.exception import ImproperConfigurationError
import asyncio
# from config import host, username, password, prod, db_name


client: AsyncIOMotorClient | None = None
db_name = None

async def init(test: bool, db_setup_data: dict, loop: asyncio.AbstractEventLoop = asyncio.get_event_loop()) -> None:
	global client
	global db_name

	conn_params = {
		'host': db_setup_data.get('host'),
		'username': db_setup_data.get('username'),
		'password': db_setup_data.get('password'),
	}
	prod = db_setup_data.get('prod')
	db_name = db_setup_data.get('db_name')
	
	if all(conn_params.values()):
		client = AsyncIOMotorClient(
    	host=f"mongodb+srv://{conn_params['host']}/?retryWrites=true&w=majority",
			username=conn_params['username'],
			password=conn_params['password'],
			uuidRepresentation='standard',
			tlsCAFile=certifi.where(),
			io_loop=loop,
		)
  
		print(await client.server_info())
	else:
		raise ImproperConfigurationError('Problem with MongoDB environment variables')

	if prod == 'false' and not test:
		db_name += '_dev'
	elif prod == 'false' and test:
		db_name += '_test'
  
	if db_name is not None:
		await init_beanie(
			database=client[db_name],
	    document_models=db_setup_data.get('document_models'),
      allow_index_dropping = True,
      recreate_views=True
    )
		return client, db_name
	else:
		raise ImproperConfigurationError('Problem with MongoDB environment variables')


async def close():
  try:
    global client
    client.close()
  except Exception as e:
    raise e



from setuptools import setup, find_packages

setup(
    name="closehaven-abc",
    version="0.1.9",
    packages=find_packages(),
    install_requires = [
        'fastapi==0.111.0',
        'pydantic==2.7.2',
        'azure-storage-blob==12.20.0',
        'certifi==2024.8.30',
        'beanie==1.26.0',
        'motor==3.5.1',
        'aio-pika==9.4.3',
        'async-timeout==4.0.3',
        'pyjwt==2.9.0',
        'uvloop==0.20.0',
        'redis==5.0.8'
    ]
)



from base_utils.exception import BadRequestError, EntityNotFoundError,UnauthorizedError

def convert_error_type(error_type: str) -> Exception:
  exception_mapping = {
    "ExpiredSignatureError": UnauthorizedError,
    "InvalidTokenError": UnauthorizedError,
    "BadRequestError": BadRequestError,
    "EntityNotFoundError": EntityNotFoundError,
    "UnauthorizedError": UnauthorizedError,
    "ZeroDivisionError": ZeroDivisionError
  }
  return exception_mapping.get(error_type, Exception)
